﻿#-------------------------------------------------------------------------------
# Author:      Krwemil
# Created:     27.03.2013
#-------------------------------------------------------------------------------

import os, subprocess, shutil, HEM_repair
from argparse import ArgumentParser
from extract_ligands import extract_ligands
from hetatomize import clean_output

def write_log(confs_dir, parameters, confs_num):
    with open(confs_dir + "confs_generation.log", "w", encoding = "utf-8") as log:
        log.write("#This is an overall log written when OMEGA finishes conformer generation for all input files\n")
        log.write("\nParameters used:\n")
        for param in parameters:
            log.write("-" + param + " " + parameters[param] + "\n")

        log.write("\nConformation counts:\n")
        counter = 0
        for name in confs_num:
            log.write(name + ": " + str(confs_num[name]) + "\n")
            counter+=confs_num[name]
        log.write("-----------------------------------------------------------------------\n")
        log.write("Total number of conformations: " + str(counter))


def main():
    #nacteni slozky s pockety daneho datasetu, vyextrahovani ligandu a jejich ulozeni do zvlastni slozky
    arg_parser = ArgumentParser()
    arg_parser.add_argument("DSFolder", help = "Slozka datasetu, ktera se ma zpracovat. \nSlozka by mela obsahovat slozku pockets, kde jsou ulozene pockety.")
    args = arg_parser.parse_args()
    confs_dir = args.DSFolder + "/conformers/"
    ligands_dir = args.DSFolder + "/ligands/"
    orderings = extract_ligands(args.DSFolder, ligands_dir)
    ligands = [x for x in os.listdir(ligands_dir) if x[:-4] in orderings]

    #slovniky pro logovani
    confs_num = dict()
    parameters_dict = dict()
    parameters_dict["confs_dir"] = confs_dir
    parameters_dict["ligands_dir"] = ligands_dir

    #inicializace cesty ke spoustecimu souboru Omegy ze souboru path.txt
    omega_path=""
    with open("path.txt","r",encoding="utf-8") as path:
        omega_path = path.readline()
        parameters_dict["OMG_path"] = omega_path.strip()
        omega_path = omega_path.strip()+" "

    #vygenerovani konformeru pro vsechny vyextrahovane ligandy
    for ligand in ligands:
##        if ligand[:-4] != "HEM":
##            continue

        if not os.path.exists(confs_dir + ligand[:-4]):
            os.makedirs(confs_dir + ligand[:-4])

        #vystrizeni zeleza z HEMu
        FE_row = ""
        if ligand[:-4] == "HEM":
            FE_row = HEM_repair.cut_FE(ligands_dir + ligand)

        inp = "-in " + ligands_dir + ligand + " "
        outp = "-out " + confs_dir + ligand[:-4] + "/" + ligand + " "

        strictatomtyping_on = False
        verbose_on = True
        #progress = "-progress dots "
        parameters_dict["strictatomtyping"] = str(strictatomtyping_on)
        parameters_dict["verbose"] = str(verbose_on)

        #parametry ovlivnujici 3D output omegy
        ewindow = "15"
        maxconfs = "500"
        maxrot = "15"
        maxtime = "180"
        rmsrange = " \"0.6,1.2,2.0\" "
        rangeIncrement = "5"
        #rmsrange = "0.8"
        #rangeIncrement = "5"
        OMG_driving_parms = {"ewindow":ewindow, "maxconfs":maxconfs, "maxrot":maxrot, "maxtime":maxtime, "rmsrange":rmsrange, "rangeIncrement":rangeIncrement}
        parameters_dict.update(OMG_driving_parms)

        strictatomtyping = ""
        if strictatomtyping_on == False:
            strictatomtyping = "-strictatomtyping false "
        verbose = ""
        if verbose_on == True:
            verbose = "-verbose "

        prefix = "-prefix " + confs_dir + ligand[:-4] + "/" + ligand[:-4] + " "

        parameters = inp + outp + strictatomtyping + verbose + prefix #+ progress

        for entry in OMG_driving_parms:
            if OMG_driving_parms[entry] != 0:
                parameters += "-" + entry + " " + OMG_driving_parms[entry] + " "

        subprocess.call(omega_path + parameters, shell = False)

        if os.path.exists(confs_dir + ligand[:-4] + "/" + ligand[:-4] + r".fail"):
            if not os.path.exists(confs_dir + "_FAILED"):
                os.makedirs(confs_dir + "_FAILED")
            shutil.copy2(confs_dir + ligand[:-4] + "/" + ligand[:-4] + r".fail", "conformers_all/_FAILED/" + ligand + r".fail")
            shutil.copy2(confs_dir + ligand[:-4] + "/" + ligand[:-4] + r".log", "conformers_all/_FAILED/" + ligand + r".log")
        else:
            with open(confs_dir + ligand[:-4] + "/" + ligand[:-4] + "_status.txt", "r", encoding = "utf-8") as log:
                lines = log.readlines()
                for line in lines:
                    if "Average Number of Conformers = " in line:
                        confs_num[ligand[:-4]] = int(float(line[len("Average Number of Conformers = "):].strip()))

        clean_output(ligand[:-4], orderings[ligand[:-4]], FE_row, confs_dir)
    #print("\n########################################################\nConformer generation finished.\nUsed parameters: " + parameters + "\n########################################################")
    write_log(confs_dir, parameters_dict,confs_num)

if __name__ == '__main__':
    main()
